% This function draws Fig. 3
% Uses the file 'ROC_N50.mat'
% Ciprian Doru Giurcaneanu 20-11-2012

function draw_fig3

% clc
clear all
close all

if exist('ROC_N_50.mat')==0,
    roc;
end

load ROC_N_50
figure
plot(log10(vec_Pfa),Pd_min,'k-.','LineWidth',2);
hold on
plot(log10(vec_Pfa),Pd_max,'k--','LineWidth',2);
plot(log10(vec_Pfa),Pd_GLRT,'k-','LineWidth',2);
plot(log10(vec_Pfa(end)),Pd_GLRT(end),'k*-','LineWidth',2, ...
                'MarkerEdgeColor','k',...
                'MarkerSize',10);
grid            
set(gca,'XTickLabel',{'-7','-6','-5','-4','-3','-2','-1','0'});
xlabel('Base-10 logarithm of probability of false alarm P_{FA}');
ylabel('Probability of detection P_D (Monte Carlo results)');
legend('ODD-MIN','ODD-MAX','GLRT','Location','NorthWest');

fprintf('*************************************************************\n');
fprintf('Experimental results:\n');
print_results(ENRdb, 10^(-4), vec_Pfa, Pd_min, Pd_max, Pd_GLRT);
print_results(ENRdb, 0.1819, vec_Pfa, Pd_min, Pd_max, Pd_GLRT);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function roc

clear all

%Init
N = 50;                              %sample size
k = 2;                               %no. of parameters
vec_Pfa = 10.^[-7:1:-1];
vec_Pfa = [vec_Pfa 0.1819];          %PFA
ENRdb = 10;
ENR = 10^(ENRdb/10);                 %ENR             
xi_min = [sqrt(ENR/2); sqrt(ENR/2)]; %ODD with worst performance
xi_max = [sqrt(ENR); 0];             %ODD with best performance
MAX_I = 10;                          %no. of realizations for H   
MAX_J = 10000;                       %no. of realizations for theta
NO_RUNS = MAX_I*MAX_J;


%Compute d for ODD
fprintf('*************************************************************\n');
fprintf('ODD: Computation of d.\n');
prec = 10^(-8);
vec_temp = zeros(size(vec_Pfa));
for i=1:length(vec_Pfa)-1,
    vec_temp(i) = pfa_inv(vec_Pfa(i), N, prec);
end
vec_d = (N*k)/(N-k)*(vec_temp.^2);
vec_d(end) = 3*k;

%Compute the threshold gam for GLRT
%GLRT: S. Kay, Vol. II, p. 345
fprintf('*************************************************************\n');
fprintf('GLRT: Computation of gam.\n');
vec_gam = zeros(size(vec_Pfa));
for i=1:length(vec_Pfa),
    vec_gam(i) = icdf('F',1-vec_Pfa(i),k,N-k);
end

count_ODD = zeros(2,length(vec_Pfa));
count_GLRT = zeros(1,length(vec_Pfa));

fprintf('*************************************************************\n');
for i=1:MAX_I,
    tau = 1+abs(randn(1,1));
    H = zeros(N,k);
    while rank(H)<2,
        H = randn(N,k);
    end
    [U,sqD,V] = svd(H,0);
    
    for sel=1:2,
        if sel==1,
            xi = xi_min; %ODD worst performance
            fprintf('Trial no. = %6i\n',(i-1)*MAX_J+1);
        else
            xi = xi_max; %ODD best performance
        end
        Htheta = sqrt(tau)*U*xi;
         
        for j=1:MAX_J,
            x = Htheta + sqrt(tau)*randn(N,1);
            PH = U*U';
            hat_tau = ( x'*(eye(N)-PH)*x )/N;
            %ODD
            t = (U'*x)/sqrt(hat_tau);                               
            %GLRT: S. Kay, Vol. II, p. 347
            Test = ((N-k)/k) * (x'*PH*x) / (x'*(eye(N)-PH)*x);     
        
            for m=1:length(vec_Pfa),
                d = vec_d(m);
                if max(abs(t)) >= sqrt(d/k),
                   count_ODD(sel,m) = count_ODD(sel,m)+1;
                end
                gam = vec_gam(m);
                if Test >= gam,
                    count_GLRT(m) = count_GLRT(m)+1;
                end
            end %m    
        end %j
    end %sel
end %i

Pd_min = count_ODD(1,:)/NO_RUNS;
Pd_max = count_ODD(2,:)/NO_RUNS;
Pd_GLRT = count_GLRT/NO_RUNS/2;

save ROC_N_50


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function print_results(ENRdb, PFA, vec_Pfa, Pd_min, Pd_max, Pd_GLRT)

ff = find(vec_Pfa==PFA);
fprintf('ODD-MAX\n');
fprintf('ENR = %3i dB, Pfa = %5.4f, Pd = %3.2f\n', ENRdb,PFA,Pd_max(ff));
fprintf('GLRT\n');
fprintf('ENR = %3i dB, Pfa = %5.4f, Pd = %3.2f\n', ENRdb,PFA,Pd_GLRT(ff));
fprintf('ODD-MIN\n');
fprintf('ENR = %3i dB, Pfa = %5.4f, Pd = %3.2f\n', ENRdb,PFA,Pd_min(ff));
fprintf('*************************************************************\n');





