% Computes E1 (E1 bar) and E2 (E2 bar) as they are shown in Table 1
% Uses the files 'p_values.mat' and 'p_values_known.mat'
% Seyed Alireza Razavi 11-01-2011
% Ciprian Doru Giurcaneanu 21-11-2012

function draw_table1

clc
clear all
close all

%Delete the file 'p_values.mat' if you want to perform all calculations
if exist('p_values.mat')==0,
    compute_E1_E2
end

%Load values for the case when the variance is known
load p_values
if exist('p_values_known.mat')==0,
    p1k = 0.1596; p2k = 0.0382; p3k = 0.0017;
else 
    load p_values_known.mat
end

%Draw Table 1
fprintf('*************************************************************\n');
fprintf('TABLE:\n');
fprintf('%6s\t','N');
for i=1:length(Nv),
    fprintf('%5i\t',Nv(i));
end
fprintf('Known\n');
print_row('E1',E1_ML,p1k);
print_row('E2',E21_ML,p2k);
print_row('E2',E22_ML,p3k);
print_row('E1 bar',E1_unbiased,p1k);
print_row('E2 bar',E21_unbiased,p2k);
print_row('E2 bar',E22_unbiased,p3k);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function compute_E1_E2     
    
maxi=100;                   % max. value of index i in Eq. (25)
maxj=100;                   % max. value of index j in Eq. (25)
Nv = [20 50 100 500 1000];  % sample sizes
k=2;                        % no. of linear parameters
C=eye(2);
A = sqrt(3);

d11=0; d12=0;
d21=2*A; d22=0;
d31=2*A; d32=2*A;

fprintf('*************************************************************\n');
fprintf('Compute E1 and E2:\n')
%Unbiased estimator
fprintf('*************************************************************\n');
fprintf('Unbiased estimator\n');
E1_unbiased = zeros(size(Nv));
E21_unbiased = zeros(size(Nv));
E22_unbiased = zeros(size(Nv));
p00 = zeros(size(Nv));
for i = 1:length(Nv),
    N = Nv(i);
    fprintf('N=%5i\n',N);
    DOF=N-k;
    E1_unbiased(i) = 1 - prob_mvt(d11,d12,A,DOF,maxi,maxj);
    p00(i) = mvtcdf(-[1;1]*A,[1; 1]*A,C,DOF);
    E21_unbiased(i) = prob_mvt(d21,d22,A,DOF,maxi,maxj);
    E22_unbiased(i) = prob_mvt(d31,d32,A,DOF,maxi,maxj);
end
[ma,w] = max(abs(E1_unbiased-(1-p00)));
fprintf('*************************************************************\n');
fprintf('The max. magnitude of the difference\n');
fprintf('when comparing the two methods for computing E1 bar is\n');
fprintf('%21.20f and corresponds to N=%5i\n', ma, Nv(w) );


%ML estimator
fprintf('*************************************************************\n');
fprintf('ML estimator\n');
E1_ML =  zeros(size(Nv));
E21_ML = zeros(size(Nv));
E22_ML =  zeros(size(Nv));
p000 = zeros(size(Nv));
for i = 1:length(Nv),
      N = Nv(i);
      fprintf('N=%5i\n',N);
      DOF=N-k;
      Ab = sqrt((N-k)/N)*A;
      E1_ML(i) = 1 - prob_mvt(d11,d12,Ab,DOF,maxi,maxj);
      p000(i) = mvtcdf(-[1;1]*Ab,[1; 1]*Ab,C,DOF);
      E21_ML(i) = prob_mvt(d21,d22,Ab,DOF,maxi,maxj);
      E22_ML(i) = prob_mvt(d31,d32,Ab,DOF,maxi,maxj);
end
[ma,w] = max(abs(E1_ML-(1-p000)));
fprintf('*************************************************************\n');
fprintf('The max. magnitude of the difference\n');
fprintf('when comparing the two methods for computing E1 is\n');
fprintf('%21.20f and corresponds to N=%5i\n', ma, Nv(w) );

save p_values E1_unbiased E21_unbiased E22_unbiased E1_ML E21_ML E22_ML Nv



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function print_row(text,vec,pp)

fprintf('%6s\t',text);
for i=1:length(vec),
    fprintf('%5.4f\t',vec(i));
end
fprintf('%5.4f\n',pp);

