% This function performs all computations which are needed for Fig. 2
% The results are saved in the file 'Pd_unknown50.mat'
% The file 'Pd_unknown50.mat' is also used to draw Fig. 4
% Seyed Alireza Razavi 11-01-2011

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function pd_uk

N=50;                  % sample size
k=2;                   % no. of linear param. 
prec = 10^(-8);        % precision (see function pfa_inv) 
% For xPFA, see the significance of x in the pfa_inv function
xPFA = [sqrt(3*(N-k)/N) pfa_inv(1e-4,N,prec)];

dof = N-k;
C=eye(2);

fprintf('*************************************************************\n');
fprintf('Find the value of x for the given PFA\n');
for ix = 1:length(xPFA)
    x = xPFA(ix);
    Pfa(ix) = 1-mvtcdf(-[x;x],[x;x],C,dof);
end

SNR      =  0 : 20;
snr      =  10.^(SNR./10);

PD_GLRT  =  zeros(length(SNR),length(xPFA));
PD_ODD_temp   =  zeros(length(SNR),length(xPFA),2);
PD_ODD_max  =  zeros(length(SNR),length(xPFA));
PD_ODD_min =  zeros(length(SNR),length(xPFA));

sigm     =  1;
maxi=100;
maxj=100;

fprintf('*************************************************************\n');
fprintf('Computations for ODD:\n');
for snri = 1 : length(snr)
    fprintf('ENR=%2i dB\n', SNR(snri));
    for pfai = 1 : length(xPFA)
        A   = xPFA(pfai);
        r   = sqrt (snr(snri));
        mcnt = 0;
        for m = [0,sqrt(0.5)]
            mcnt = mcnt + 1;
            A_scaled =  m * r;
            B_scaled = sqrt ( r^2 - A_scaled^2);
            PD_ODD_temp ( snri , pfai , mcnt ) = 1 - prob_mvt (A_scaled,B_scaled,A,N,maxi,maxj); 
        end
        PD_ODD_max ( snri , pfai ) = PD_ODD_temp (snri,pfai,1);
        PD_ODD_min ( snri , pfai ) = PD_ODD_temp (snri,pfai,2);
    end
end

fprintf('*************************************************************\n');
fprintf('Computations for GLRT\n');
for snri = 1 : length(snr)
    for pfai = 1 : length(xPFA)
        PD_GLRT ( snri , pfai ) = 1 - ncfcdf ( icdf('F',1-Pfa(pfai),2,N-2) , 2 , N-2 , snr(snri) );
    end
end
fprintf('*************************************************************\n');

save Pd_unknown50 PD_ODD_min PD_ODD_max PD_GLRT N xPFA Pfa SNR
